@@ -1,4 +1,6 @@
 {-# LANGUAGE NoImplicitPrelude #-}
+{-# LANGUAGE ViewPatterns      #-}
+{-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE FlexibleContexts #-}
 {-# OPTIONS_GHC -fno-warn-unused-do-bind #-}
@@ -33,6 +35,7 @@ import           Data.List              (foldl', intercalate)
 import           Data.List.Split        (splitOn, splitWhen, wordsBy)
 import qualified Data.Map               as Map
 import           Data.Maybe
+import qualified Data.Text              as T
 import           System.Environment     (getEnvironment)
 import           Text.CSL.Compat.Pandoc (readLaTeX)
 import           Text.CSL.Exception     (CiteprocException (ErrorReadingBib, ErrorReadingBibFile))
@@ -41,7 +44,8 @@ import           Text.CSL.Reference
 import           Text.CSL.Style         (Agent (..), emptyAgent, CslTerm (..),
                                          Formatted (..), Locale (..))
 import           Text.CSL.Util          (onBlocks, protectCase, safeRead,
-                                         splitStrWhen, trim, unTitlecase)
+                                         splitStrWhen, trim, unTitlecase,
+                                         addSpaceAfterPeriod)
 import           Text.Pandoc.Definition
 import qualified Text.Pandoc.UTF8       as UTF8
 import qualified Text.Pandoc.Walk       as Walk
@@ -59,7 +63,7 @@ adjustSpans _ (Span ("",[],[]) xs) = xs
 adjustSpans lang (RawInline (Format "latex") s)
   | s == "\\hyphen" || s == "\\hyphen " = [Str "-"]
   | otherwise = Walk.walk (concatMap (adjustSpans lang))
-                $ parseRawLaTeX lang s
+                $ parseRawLaTeX lang (T.unpack s)
 adjustSpans _ x = [x]
 
 parseRawLaTeX :: Lang -> String -> [Inline]
@@ -80,7 +84,7 @@ parseRawLaTeX lang ('\\':xs) =
          -- ... both should be nestable & should work in year fields
          f "autocap"    ils    = ils  -- TODO: should work in year fields
          f "textnormal" ils    = [Span ("",["nodecor"],[]) ils]
-         f "bibstring" [Str s] = [Str $ resolveKey' lang s]
+         f "bibstring" [Str s] = [Str $ T.pack $ resolveKey' lang $ T.unpack s]
          f _            ils    = [Span nullAttr ils]
 parseRawLaTeX _ _ = []
 
@@ -99,7 +103,8 @@ getLangFromEnv :: IO Lang
 getLangFromEnv = do
   env <- getEnvironment
   return $ case lookup "LANG" env of
-                  Just x  -> case splitWhen (\c -> c == '.' || c == '_' || c == '-') x of
+                  Just x  -> case splitWhen (\c -> c == '_' || c == '-')
+                                  (takeWhile (/='.') x) of
                                    (w:z:_) -> Lang w z
                                    [w]     | not (null w) -> Lang w mempty
                                    _       -> Lang "en" "US"
@@ -108,7 +113,7 @@ getLangFromEnv = do
 -- | Parse a BibTeX or BibLaTeX file into a list of 'Reference's.
 -- The first parameter is a predicate to filter identifiers.
 -- If the second parameter is true, the file will be treated as
--- BibTeX; otherwse as BibLaTeX.  If the third parameter is
+-- BibTeX; otherwise as BibLaTeX.  If the third parameter is
 -- true, an "untitlecase" transformation will be performed.
 readBibtex :: (String -> Bool) -> Bool -> Bool -> FilePath -> IO [Reference]
 readBibtex idpred isBibtex caseTransform f = do
@@ -197,7 +202,7 @@ fieldName =
   resolveAlias . map toLower <$> many1 (letter <|> digit <|> oneOf "-_:+")
 
 isBibtexKeyChar :: Char -> Bool
-isBibtexKeyChar c = isAlphaNum c || c `elem` ".:;?!`'()/*@_+=-[]*&"
+isBibtexKeyChar c = isAlphaNum c || c `elem` (".:;?!`'()/*@_+=-[]*&" :: String)
 
 bibItem :: BibParser Item
 bibItem = do
@@ -367,7 +372,7 @@ data Lang = Lang String String  -- e.g. "en" "US"
 
 -- | Prints a 'Lang' in BCP 47 format.
 langToLocale :: Lang -> String
-langToLocale (Lang x y) = x ++ ('-':y)
+langToLocale (Lang x y) = x ++ (if null y then [] else '-':y)
 
 -- Biblatex Localization Keys (see Biblatex manual)
 -- Currently we only map a subset likely to be used in Biblatex *databases*
@@ -375,7 +380,7 @@ langToLocale (Lang x y) = x ++ ('-':y)
 
 resolveKey :: Lang -> Formatted -> Formatted
 resolveKey lang (Formatted ils) = Formatted (Walk.walk go ils)
-  where go (Str s) = Str $ resolveKey' lang s
+  where go (Str s) = Str $ T.pack $ resolveKey' lang $ T.unpack s
         go x       = x
 
 -- biblatex localization keys, from files at
@@ -875,7 +880,7 @@ parseMonth s =
          "oct" -> Just 10
          "nov" -> Just 11
          "dec" -> Just 12
-         _     -> safeRead s
+         _     -> safeRead $ T.pack s
 
 data BibState = BibState{
            untitlecase    :: Bool
@@ -885,7 +890,7 @@ data BibState = BibState{
 type Bib = RWST Item () BibState Maybe
 
 notFound :: String -> Bib a
-notFound f = fail $ f ++ " not found"
+notFound f = Prelude.fail $ f ++ " not found"
 
 getField :: String -> Bib Formatted
 getField f = do
@@ -949,20 +954,20 @@ getOldDates prefix = do
   year' <- fixLeadingDash <$> getRawField (prefix ++ "year") <|> return ""
   month' <- (parseMonth
               <$> getRawField (prefix ++ "month")) <|> return Nothing
-  day' <- (safeRead <$> getRawField (prefix ++ "day")) <|> return Nothing
+  day' <- (safeRead . T.pack <$> getRawField (prefix ++ "day")) <|> return Nothing
   endyear' <- (fixLeadingDash <$> getRawField (prefix ++ "endyear"))
             <|> return ""
   endmonth' <- (parseMonth <$> getRawField (prefix ++ "endmonth"))
             <|> return Nothing
-  endday' <- (safeRead <$> getRawField (prefix ++ "endday")) <|> return Nothing
-  let start' = RefDate { year   = safeRead year'
+  endday' <- (safeRead . T.pack <$> getRawField (prefix ++ "endday")) <|> return Nothing
+  let start' = RefDate { year   = safeRead $ T.pack year'
                        , month  = month'
                        , season = Nothing
                        , day    = day'
                        , other  = Literal $ if isNumber year' then "" else year'
                        , circa  = False
                        }
-  let end' = RefDate { year     = safeRead endyear'
+  let end' = RefDate { year     = safeRead $ T.pack endyear'
                        , month  = endmonth'
                        , day    = endday'
                        , season = Nothing
@@ -1009,7 +1014,8 @@ toLiteralList _ = mzero
 
 toAuthorList :: Options -> [Block] -> Bib [Agent]
 toAuthorList opts [Para xs] =
-  filter (/= emptyAgent) <$> mapM (toAuthor opts) (splitByAnd xs)
+  filter (/= emptyAgent) <$> mapM (toAuthor opts . addSpaceAfterPeriod)
+                                    (splitByAnd xs)
 toAuthorList opts [Plain xs] = toAuthorList opts [Para xs]
 toAuthorList _ _ = mzero
 
@@ -1037,7 +1043,7 @@ toAuthor _ [Span ("",[],[]) ils] =
           , parseNames      = False
           }
  -- extended BibLaTeX name format - see #266
-toAuthor _ ils@(Str ys:_) | '=' `elem` ys = do
+toAuthor _ ils@(Str ys:_) | T.any (== '=') ys = do
   let commaParts = splitWhen (== Str ",")
                    $ splitStrWhen (\c -> c == ',' || c == '=' || c == '\160')
                    $ ils
@@ -1111,7 +1117,7 @@ toAuthor opts ils = do
           }
 
 isCapitalized :: [Inline] -> Bool
-isCapitalized (Str (c:cs) : rest)
+isCapitalized (Str (T.uncons -> Just (c,cs)) : rest)
   | isUpper c = True
   | isDigit c = isCapitalized (Str cs : rest)
   | otherwise = False
@@ -1217,9 +1223,9 @@ concatWith sep = Formatted . foldl' go mempty . map unFormatted
         go accum s  = case reverse accum of
                            []    -> s
                            (Str x:_)
-                             | not (null x) && last x `elem` "!?.,:;"
+                             | not (T.null x) && T.last x `elem` ("!?.,:;" :: String)
                                           -> accum ++ (Space : s)
-                           _     -> accum ++ (Str [sep] : Space : s)
+                           _     -> accum ++ (Str (T.singleton sep) : Space : s)
 
 type Options = [(String, String)]
 
@@ -1267,6 +1273,7 @@ itemToReference lang locale bibtex caseTransform = bib $ do
        "booklet"         -> (Pamphlet,mempty)
        "bookinbook"      -> (Chapter,mempty)
        "collection"      -> (Book,mempty)
+       "dataset"         -> (Dataset,mempty)
        "electronic"      -> (Webpage,mempty)
        "inbook"          -> (Chapter,mempty)
        "incollection"    -> (Chapter,mempty)
@@ -1274,7 +1281,7 @@ itemToReference lang locale bibtex caseTransform = bib $ do
        "inproceedings"   -> (PaperConference,mempty)
        "manual"          -> (Book,mempty)
        "mastersthesis"   -> (Thesis, if reftype' == mempty
-                                        then Formatted [Str $ resolveKey' lang "mathesis"]
+                                        then Formatted [Str $ T.pack $ resolveKey' lang "mathesis"]
                                         else reftype')
        "misc"            -> (NoType,mempty)
        "mvbook"          -> (Book,mempty)
@@ -1288,11 +1295,12 @@ itemToReference lang locale bibtex caseTransform = bib $ do
          | st == "newspaper" -> (ArticleNewspaper,mempty)
          | otherwise         -> (ArticleJournal,mempty)
        "phdthesis"       -> (Thesis, if reftype' == mempty
-                                        then Formatted [Str $ resolveKey' lang "phdthesis"]
+                                        then Formatted [Str $ T.pack $ resolveKey' lang "phdthesis"]
                                         else reftype')
        "proceedings"     -> (Book,mempty)
        "reference"       -> (Book,mempty)
        "report"          -> (Report,mempty)
+       "software"        -> (Book,mempty)         -- no "software" type in CSL
        "suppbook"        -> (Chapter,mempty)
        "suppcollection"  -> (Chapter,mempty)
        "suppperiodical"
@@ -1316,7 +1324,6 @@ itemToReference lang locale bibtex caseTransform = bib $ do
        "music"           -> (Song,mempty)         -- for musical *recordings*
        "performance"     -> (Speech,mempty)
        "review"          -> (Review,mempty)       -- or "review-book" ?
-       "software"        -> (Book,mempty)         -- for lack of any better match
        "standard"        -> (Legislation,mempty)
        "video"           -> (MotionPicture,mempty)
        -- biblatex-apa:
@@ -1411,9 +1418,9 @@ itemToReference lang locale bibtex caseTransform = bib $ do
                         <|> getPeriodicalTitle "shortjournal"
                         <|> return mempty
   -- change numerical series title to e.g. 'series 3'
-  let fixSeriesTitle (Formatted [Str xs]) | all isDigit xs =
-         Formatted [Str (ordinalize locale xs),
-                    Space, Str (resolveKey' lang "ser.")]
+  let fixSeriesTitle (Formatted [Str xs]) | T.all isDigit xs =
+         Formatted [Str (T.pack $ ordinalize locale $ T.unpack xs),
+                    Space, Str (T.pack $ resolveKey' lang "ser.")]
       fixSeriesTitle x = x
   seriesTitle' <- (fixSeriesTitle . resolveKey lang) <$>
                       getTitle "series" <|> return mempty
@@ -1531,12 +1538,12 @@ itemToReference lang locale bibtex caseTransform = bib $ do
                          _ -> return mempty
                  )
 
-  let convertEnDash (Str s) = Str (map (\c -> if c == '–' then '-' else c) s)
+  let convertEnDash (Str s) = Str (T.map (\c -> if c == '–' then '-' else c) s)
       convertEnDash x       = x
 
   let takeDigits (Str xs : _) =
-         case takeWhile isDigit xs of
-              [] -> []
+         case T.takeWhile isDigit xs of
+              "" -> []
               ds -> [Str ds]
       takeDigits x             = x
 
