@@ -62,11 +62,11 @@ import           Control.Monad       (guard, mplus, msum)
 import           Data.Aeson          hiding (Value)
 import qualified Data.Aeson          as Aeson
 import           Data.Aeson.Types    (Parser)
-import           Data.Char           (isDigit, toLower)
+import           Data.Char           (isDigit, toLower, isPunctuation)
 import           Data.Either         (lefts, rights)
 import           Data.Generics       hiding (Generic)
 import qualified Data.HashMap.Strict as H
-import           Data.List           (findIndex, elemIndex)
+import           Data.List           (find, elemIndex)
 import           Data.List.Split     (splitWhen)
 import           Data.Maybe          (fromMaybe, isNothing)
 import           Data.String
@@ -80,7 +80,7 @@ import           Text.CSL.Style      hiding (Number)
 import           Text.CSL.Util       (camelize, capitalize, inlinesToString,
                                       mapping', parseBool, parseInt, parseMaybeInt,
                                       parseString, readNum, safeRead, trim,
-                                      uncamelize, AddYaml(..))
+                                      uncamelize, AddYaml(..), splitStrWhen)
 import           Text.Pandoc         (Inline (Str))
 import qualified Text.Parsec         as P
 import qualified Text.Parsec.String  as P
@@ -303,8 +303,8 @@ handleLiteral d@(RefDate Nothing Nothing Nothing Nothing (Literal xs) b)
   = case splitWhen (=='_') xs of
          [x,y] | all isDigit x && all isDigit y &&
                  not (null x) ->
-                 [RefDate (safeRead x) Nothing Nothing Nothing mempty b,
-                  RefDate (safeRead y) Nothing Nothing Nothing mempty b]
+                 [RefDate (safeRead $ T.pack x) Nothing Nothing Nothing mempty b,
+                  RefDate (safeRead $ T.pack y) Nothing Nothing Nothing mempty b]
          _ -> [d]
 handleLiteral d = [d]
 
@@ -360,11 +360,11 @@ instance FromJSON RefType where
   -- found in one of the test cases:
   parseJSON (String "film") = return MotionPicture
   parseJSON (String t) =
-    safeRead (capitalize . camelize . T.unpack $ t) <|>
+    safeRead (T.pack . capitalize . camelize . T.unpack $ t) <|>
     fail ("'" ++ T.unpack t ++ "' is not a valid reference type")
   parseJSON v@(Array _) =
     fmap (capitalize . camelize . inlinesToString) (parseJSON v) >>= \t ->
-      safeRead t <|>
+      safeRead (T.pack t) <|>
        fail ("'" ++ t ++ "' is not a valid reference type")
   parseJSON _ = fail "Could not parse RefType"
 
@@ -382,7 +382,8 @@ handleSpecialCases "personal-communication" = "personal_communication"
 handleSpecialCases "legal-case"             = "legal_case"
 handleSpecialCases x                        = x
 
-newtype CNum = CNum { unCNum :: Int } deriving ( Show, Read, Eq, Num, Typeable, Data, Generic )
+newtype CNum = CNum { unCNum :: Int }
+  deriving ( Show, Read, Eq, Ord, Num, Typeable, Data, Generic )
 
 instance FromJSON CNum where
   parseJSON x = CNum `fmap` parseInt x
@@ -492,7 +493,7 @@ data Reference =
 instance FromJSON Reference where
   parseJSON (Object v') = do
      v <- parseSuppFields v' <|> return v'
-     addPageFirst <$> (Reference <$>
+     (Reference <$>
        v .:? "id" .!= "" <*>
        v .:? "other-ids" .!= [] <*>
        v .:? "type" .!= NoType <*>
@@ -569,15 +570,6 @@ instance FromJSON Reference where
        v .:? "citation-number" .!= CNum 0 <*>
        ((v .: "first-reference-note-number" >>= parseInt) <|> return 0) <*>
        v .:? "citation-label" .!= mempty)
-    where takeFirstNum (Formatted (Str xs : _)) =
-            case takeWhile isDigit xs of
-                   [] -> mempty
-                   ds -> Formatted [Str ds]
-          takeFirstNum x = x
-          addPageFirst ref = if pageFirst ref == mempty && page ref /= mempty
-                                then ref{ pageFirst =
-                                            takeFirstNum (page ref) }
-                                else ref
   parseJSON _ = fail "Could not parse Reference"
 
 -- Syntax for adding supplementary fields in note variable
@@ -877,8 +869,8 @@ numericVars = [ "edition", "volume", "number-of-volumes", "number", "issue", "ci
 
 getReference :: [Reference] -> Cite -> Maybe Reference
 getReference  rs c
-    = case (hasId (citeId c)) `findIndex` rs of
-        Just i  -> Just $ setPageFirst $ rs !! i
+    = case (hasId (citeId c)) `find` rs of
+        Just r  -> Just $ setPageFirst r
         Nothing -> Nothing
   where hasId :: String -> Reference -> Bool
         hasId ident r = ident `elem` (map unLiteral (refId r : refOtherIds r))
@@ -955,7 +947,8 @@ processCites rs cs
 setPageFirst :: Reference -> Reference
 setPageFirst ref =
   let Formatted ils = page ref
-      ils' = takeWhile (\i -> i /= Str "–" && i /= Str "-") ils
+      ils' = takeWhile (\i -> i /= Str "–" && i /= Str "-") $
+              splitStrWhen isPunctuation ils
   in  if ils == ils'
          then ref
          else ref{ pageFirst = Formatted ils' }
@@ -1024,18 +1017,18 @@ isoDate = P.try $ do
     extended <- if extyear
                    then P.many P.digit
                    else return []
-    return $ case safeRead (sign ++ rest ++ extended) of
+    return $ case safeRead (T.pack $ sign ++ rest ++ extended) of
                     Just x | x <= 0 -> Just (x - 1)  -- 0 = -1 AD
                     x               -> x
   m' <- P.option Nothing $ Just <$> P.try (P.char '-' >> P.many1 P.digit)
-  (m,s) <- case m' >>= safeRead of
+  (m,s) <- case m' >>= safeRead . T.pack of
                    Just (n::Int)
                           | n >= 1 && n <= 12  -> return (Just n, Nothing)
                           | n >= 13 && n <= 16 -> return (Nothing, pseudoMonthToSeason n)
                           | n >= 21 && n <= 24 -> return (Nothing, pseudoMonthToSeason n)
                    Nothing | isNothing m' -> return (Nothing, Nothing)
                    _ -> fail "Improper month"
-  d <- P.option Nothing $ safeRead <$> P.try (P.char '-' >> P.many1 P.digit)
+  d <- P.option Nothing $ safeRead . T.pack <$> P.try (P.char '-' >> P.many1 P.digit)
   guard $ case d of
            Nothing -> True
            Just (n::Int) | n >= 1 && n <= 31 -> True
@@ -1059,7 +1052,7 @@ rawDateOld = do
   let pmonth = P.try $ do
         xs <- P.many1 P.letter <|> P.many1 P.digit
         if all isDigit xs
-           then case safeRead xs of
+           then case safeRead (T.pack xs) of
                       Just (n::Int) | n >= 1 && n <= 12 -> return (Just n)
                       _ -> fail "Improper month"
            else case elemIndex (map toLower $ take 3 xs) months of
@@ -1075,10 +1068,10 @@ rawDateOld = do
              _       -> fail "Improper season"
   let pday = P.try $ do
         xs <- P.many1 P.digit
-        case safeRead xs of
+        case safeRead (T.pack xs) of
              Just (n::Int) | n >= 1 && n <= 31 -> return (Just n)
              _ -> fail "Improper day"
-  let pyear = safeRead <$> P.many1 P.digit
+  let pyear = safeRead . T.pack <$> P.many1 P.digit
   let sep = P.oneOf [' ','/',','] >> P.spaces
   let rangesep = P.try $ P.spaces >> P.char '-' >> P.spaces
   let refDate = RefDate Nothing Nothing Nothing Nothing mempty False
