@@ -3,6 +3,7 @@
 {-# LANGUAGE DeriveDataTypeable         #-}
 {-# LANGUAGE DeriveGeneric              #-}
 {-# LANGUAGE FlexibleInstances          #-}
+{-# LANGUAGE ViewPatterns               #-}
 
 {-# LANGUAGE MultiParamTypeClasses      #-}
 {-# LANGUAGE OverloadedStrings          #-}
@@ -100,7 +101,7 @@ import           Control.Monad          (mplus)
 import           Data.Aeson             hiding (Number)
 import qualified Data.Aeson             as Aeson
 import           Data.Aeson.Types       (Pair)
-import           Data.Char              (isLetter, isPunctuation, isUpper, toLower)
+import           Data.Char              (isLetter, isPunctuation, isUpper, toLower, isDigit)
 import qualified Data.Char              as Char
 import           Data.Generics          (Data, Typeable)
 import           Data.List              (intercalate, intersperse, isInfixOf,
@@ -118,7 +119,7 @@ import           Text.CSL.Util          (orIfNull, headInline, initInline,
                                          lastInline, mapping', mb, parseBool,
                                          parseString, query, splitStrWhen,
                                          tailInline, trimr, (.#:), (.#?),
-                                         AddYaml(..))
+                                         AddYaml(..), addSpaceAfterPeriod)
 import qualified Text.Pandoc.Builder    as B
 import           Text.Pandoc.Definition hiding (Citation, Cite)
 import qualified Text.Pandoc.Walk       as Walk
@@ -147,7 +148,7 @@ readCSLString s = Walk.walk handleSmallCapsSpans
   -- this is needed for versions of pandoc that don't turn
   -- a span with font-variant:small-caps into a SmallCaps element:
   where handleSmallCapsSpans (Span ("",[],[("style",sty)]) ils)
-            | filter (`notElem` (" \t;" :: String)) sty == "font-variant:small-caps" =
+            | T.filter (`notElem` (" \t;" :: String)) sty == "font-variant:small-caps" =
               SmallCaps ils
         handleSmallCapsSpans x = x
 
@@ -245,7 +246,7 @@ instance Walk.Walkable Formatted Formatted where
 
 toStr :: String -> [Inline]
 toStr = intercalate [Str "\n"] .
-        map (B.toList . B.text . tweak . fromEntities) .
+        map (B.toList . B.text . T.pack . tweak . T.unpack . fromEntities . T.pack) .
         splitWhen (=='\n')
     where
       tweak ('«':' ':xs) = "«\8239" ++ tweak xs
@@ -491,7 +492,13 @@ compare' x y
         (_  ,'-':_)   -> GT
         _             -> comp (normalize x) (normalize y)
       where
-        normalize = map (\c -> if c == ',' || c == '.' then ' ' else c) .
+        -- we zero pad numbers so they're sorted numerically, see #399
+        zeropad [] = []
+        zeropad xs = if all isDigit xs
+                        then replicate (10 - length xs) '0' ++ xs
+                        else xs
+        normalize = zeropad .
+                    map (\c -> if c == ',' || c == '.' then ' ' else c) .
                     filter (\c -> c == ',' ||
                                   not (isPunctuation c || Char.isSpace c
                                       -- ayn/hamza in transliterated arabic:
@@ -676,6 +683,7 @@ data Output
     = ONull
     | OSpace
     | OPan    [Inline]
+    | OStatus [Inline]
     | ODel     String                                   -- ^ A delimiter string.
     | OStr     String             Formatting            -- ^ A simple 'String'
     | OErr     CiteprocError                            -- ^ Warning message
@@ -818,7 +826,9 @@ removeQuoted (Formatted ils) = Formatted (go ils)
 
 instance FromJSON Agent where
   parseJSON (Object v) = nameTransform <$> (Agent <$>
-              (v .: "given" <|> ((map Formatted . wordsBy isSpace . unFormatted) <$> v .: "given") <|> pure []) <*>
+              (v .: "given" <|> ((map Formatted . wordsBy isSpace .
+                                  addSpaceAfterPeriod . unFormatted) <$>
+                                  v .: "given") <|> pure []) <*>
               v .:?  "dropping-particle" .!= mempty <*>
               v .:? "non-dropping-particle" .!= mempty <*>
               (removeQuoted <$> (v .:? "family" .!= mempty)) <*>
@@ -889,7 +899,7 @@ droppingPartTransform ag
   | otherwise = ag
 
 startWithCapital' :: Inline -> Bool
-startWithCapital' (Str (c:_)) = isUpper c && isLetter c
+startWithCapital' (Str (T.uncons -> Just (c,_))) = isUpper c && isLetter c
 startWithCapital' _           = False
 
 startWithCapital :: Formatted -> Bool
